"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const defaultTranslation = require("../assets/en_us.json");
/**
 * Converts a Minecraft chat component to a formatted string.
 * */
function format(component, options = {}) {
    return formatString(convert(component, options), options.useAnsiCodes);
}
exports.format = format;
/**
 * Applies translations to a [Chat component](https://wiki.vg/Chat) and converts
 * it to `StringComponent`.
*/
function convert(component, options = {}) {
    if (!options.translation)
        options.translation = defaultTranslation;
    if (options.strict == null)
        options.strict = true;
    if (typeof component == "string")
        return { text: component };
    if (component.extra) {
        component.extra = component.extra.map(x => convert(x, options));
    }
    if (options.stripNonText) {
        delete component.hoverEvent;
        delete component.clickEvent;
        delete component.insertion;
    }
    if ("translate" in component) {
        const { translate, with: _with, ...rest } = component;
        const translation = options.translation[translate];
        if (!translation)
            if (options.strict)
                throw new Error(`Couldn't find translation for ${translate}`);
            else
                return { text: "[Missing translation]", ...rest };
        const match = translation.match(/%([0-9]\$)?s/g);
        if (!match)
            return { text: translation, ...rest };
        const order = match.map((x, i) => x.length == 2 ? i : parseInt(x[1]) - 1);
        const parts = translation.split(/%([0-9]\$)?s/);
        const extra = parts.map((x, i) => {
            if (i % 2 == 0)
                return { text: x };
            else
                return convert(_with[order[(i / 2) | 0]], options);
        }).filter(x => x.text.length > 0 || x.extra);
        if (rest.extra)
            extra.push(...rest.extra);
        return { text: "", ...rest, extra };
    }
    else {
        return component;
    }
}
exports.convert = convert;
/** Flattens a nested `StringComponent` to an array. */
function flatten(component) {
    const { text, extra, ...rest } = component;
    const array = [{ text, ...rest }];
    if (extra)
        array.push(...flattenArray(extra.map(x => flatten(x).map(x => ({
            ...x, ...rest, ...x
        })))));
    return array;
}
exports.flatten = flatten;
/** Converts a `StringComponent` to plain text and can format it using ANSI codes. */
function formatString(component, useAnsiCodes = false) {
    let text = flatten(component).map(c => {
        if (!useAnsiCodes)
            return c.text;
        let text = colorToAnsiCode(c.color);
        if (c.bold)
            text += "\x1b[1m";
        if (c.italic)
            text += "\x1b[3m";
        if (c.underlined)
            text += "\x1b[4m";
        if (c.strikethrough)
            text += "\x1b[9m";
        return text + c.text + "\x1b[0m";
    }).join("");
    return text.split(/§(.)/).map((t, i) => {
        if (i % 2 == 0) {
            return t;
        }
        else
            switch (t) {
                case "l": return "\x1b[1m";
                case "m": return "\x1b[9m";
                case "n": return "\x1b[4m";
                case "o": return "\x1b[3m";
                case "r": return "\x1b[0m";
                case "0": return "\x1b[38;2;0;0;0m";
                case "1": return "\x1b[38;2;0;0;170m";
                case "2": return "\x1b[38;2;0;170;0m";
                case "3": return "\x1b[38;2;0;170;170m";
                case "4": return "\x1b[38;2;170;0;0m";
                case "5": return "\x1b[38;2;170;0;170m";
                case "6": return "\x1b[38;2;255;170;0m";
                case "7": return "\x1b[38;2;170;170;170m";
                case "8": return "\x1b[38;2;85;85;85m";
                case "9": return "\x1b[38;2;85;85;255m";
                case "a": return "\x1b[38;2;85;255;85m";
                case "b": return "\x1b[38;2;85;255;255m";
                case "c": return "\x1b[38;2;255;85;85m";
                case "d": return "\x1b[38;2;255;85;255m";
                case "e": return "\x1b[38;2;255;255;85m";
                case "f": return "\x1b[38;2;255;255;255m";
            }
    }).join("");
}
exports.formatString = formatString;
function colorToAnsiCode(color) {
    let code = "\x1b[38;2;";
    switch (color) {
        case "black":
            code += "0;0;0";
            break;
        case "dark_blue":
            code += "0;0;170";
            break;
        case "dark_green":
            code += "0;170;0";
            break;
        case "dark_aqua":
            code += "0;170;170";
            break;
        case "dark_red":
            code += "170;0;0";
            break;
        case "dark_purple":
            code += "170;0;170";
            break;
        case "gold":
            code += "255;170;0";
            break;
        case "gray":
            code += "170;170;170";
            break;
        case "dark_gray":
            code += "85;85;85";
            break;
        case "blue":
            code += "85;85;255";
            break;
        case "green":
            code += "85;255;85";
            break;
        case "aqua":
            code += "85;255;255";
            break;
        case "red":
            code += "255;85;85";
            break;
        case "light_purple":
            code += "255;85;255";
            break;
        case "yellow":
            code += "255;255;85";
            break;
        case "white":
            code += "255;255;255";
            break;
        default: code = "\x1b[0";
    }
    return code + "m";
}
function flattenArray(array) {
    return [].concat(...array);
}
/** @deprecated Use `format(convert(component))` instead */
function chatToText(component, translation) {
    return format(component, { translation });
}
exports.chatToText = chatToText;
